@@ -1,3 +1,6 @@
+import pydantic
+import pytest
+
 from inline_snapshot import snapshot
 from inline_snapshot.testing import Example
 
@@ -24,7 +27,7 @@ def test_pydantic():
         ["--inline-snapshot=create"],
         changed_files=snapshot(
             {
-                "test_something.py": """\
+                "tests/test_something.py": """\
 
 from pydantic import BaseModel
 from inline_snapshot import snapshot
@@ -67,7 +70,7 @@ def test():
         ["--inline-snapshot=create"],
         changed_files=snapshot(
             {
-                "test_something.py": """\
+                "tests/test_something.py": """\
 from inline_snapshot import snapshot
 from pydantic import BaseModel,Field
 
@@ -105,7 +108,7 @@ def test_something():
         ["--inline-snapshot=update"],
         changed_files=snapshot(
             {
-                "test_something.py": """\
+                "tests/test_something.py": """\
 from inline_snapshot import snapshot,Is
 from dataclasses import dataclass,field
 from pydantic import BaseModel,Field
@@ -162,7 +165,7 @@ def test_something():
         ["--inline-snapshot=fix"],
         changed_files=snapshot(
             {
-                "test_something.py": """\
+                "tests/test_something.py": """\
 from inline_snapshot import snapshot
 from pydantic import BaseModel
 
@@ -181,3 +184,46 @@ def test_something():
         ),
         returncode=1,
     )
+
+
+@pytest.mark.skipif(
+    pydantic.version.VERSION.startswith("1."),
+    reason="pydantic 1 can not compare C[int]() with C()",
+)
+def test_pydantic_generic_class():
+    Example(
+        """\
+from typing import Generic, TypeVar
+from inline_snapshot import snapshot
+from pydantic import BaseModel
+
+I=TypeVar("I")
+class C(BaseModel,Generic[I]):
+    a:int
+
+def test_a():
+    c=C[int](a=5)
+
+    assert c == snapshot()
+"""
+    ).run_inline(
+        ["--inline-snapshot=create"],
+        changed_files=snapshot(
+            {
+                "tests/test_something.py": """\
+from typing import Generic, TypeVar
+from inline_snapshot import snapshot
+from pydantic import BaseModel
+
+I=TypeVar("I")
+class C(BaseModel,Generic[I]):
+    a:int
+
+def test_a():
+    c=C[int](a=5)
+
+    assert c == snapshot(C(a=5))
+"""
+            }
+        ),
+    ).run_inline()
